
import de.tum.in.net.WSNDataFramework.WSNApp;
import de.tum.in.net.WSNDataFramework.Modules.Contiki.WSNContikiModule;
import de.tum.in.net.WSNDataFramework.Modules.Cosm.WSNCosmModule;
import de.tum.in.net.WSNDataFramework.Modules.FileLogger.WSNFileLoggerModule;
import de.tum.in.net.WSNDataFramework.Modules.HTTPServer.WSNHTTPServerModule;
import de.tum.in.net.WSNDataFramework.Modules.SSHServer.WSNSSHServerModule;
import de.tum.in.net.WSNDataFramework.Modules.TinyOS.TinyOSHelperModule;
import de.tum.in.net.WSNDataFramework.Protocols.IPFIXEnricherProtocol;
import de.tum.in.net.WSNDataFramework.Protocols.IPFIXProtocol;
import de.tum.in.net.WSNDataFramework.Protocols.PhilippSSHProtocol;
import de.tum.in.net.WSNDataFramework.Protocols.TUMI8Adapter;
import de.tum.in.net.WSNDataFramework.Protocols.TinyIPFIXProtocol;
import de.tum.in.net.WSNDataFramework.Protocols.TinyOSDTLSThomasDriver;



public class WSNFramework {

	/**
	 * Example WSN-Data-Framework Application
	 * 
	 * @param args
	 */
	public static void main(String[] args) {

		try {
			/*** create a WSN instance ***/
			//WSNApp app = new WSNApp(); 										// WSNApp with default paths
			WSNApp app = new WSNApp("/home/tinyos/Desktop/database", "/tmp");	// WSNApp with specific storage paths
			//			app.addModule(new TestWSNDriverModule());							// Test WSN-Driver
			app.setDriver(														// TinyOS WSN-Driver
					//new TinyOSDriver()                     //in if no OPAL in use
					new TinyOSDTLSThomasDriver()			// for Thomas' DTLS
					);
			app.setProtocolStack(												// Protocol Stack, attaches the actual WSN to the App
					new PhilippSSHProtocol(),
					new TinyIPFIXProtocol(),
					new IPFIXProtocol(),
					new IPFIXEnricherProtocol("conf/tinyIPFIX-conf.xml"),
					new TUMI8Adapter()
					);
			System.out.println("WSNApp up");


			/*** add modules ***/
			// attach the actual WSN
			app.addModule(new TinyOSHelperModule("/opt/tinyos-2.1.1"));											// TinyOS GUI
			app.addModule(new WSNContikiModule());																// Contiki

			// User Interface
			app.addModule(new WSNHTTPServerModule(8000, "html"));												// HTTP/HTML GUI

			WSNSSHServerModule sshServer = new WSNSSHServerModule(2222);										// SSH UI
			sshServer.getAuthenticator().addPassword(null, null);												// allowing any username+password combination to access it
			app.addModule(sshServer);


			// Logger
			app.addModule(new WSNFileLoggerModule());															// TXT logging
			app.addModule(new WSNCosmModule()); //("ratatatata","fetY7Y1WCQNWGMEhCIbdZd_HiRbP-ODVxJIcNvppzVA"));																	// Cosm Upload



			/*** wait for the WSN to shutdown ***/
			System.out.println("waiting for WSNApp to shut down");
			try {
				app.waitForShutdown();
			} catch(Exception e) {
			}
			System.out.println("WSN down");



		} catch (Exception e) {
			System.out.println("CRASH: " + e.getMessage());
			e.printStackTrace();
		}
	}

}
