package de.tum.in.net.WSNDataFramework.Modules.HTTPServer;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;

import org.json.simple.JSONValue;

import de.tum.in.net.WSNDataFramework.WSNModule;
import de.tum.in.net.WSNDataFramework.WSNProtocol;






/**
 * Index controller.
 * 
 * @author André Freitag
 *
 */
public class WSNHTTPIndexController extends WSNHTTPController {

	/**
	 * Index action.
	 * 
	 * @param request
	 * @param response
	 */
	public void indexAction(HTTPRequest request, HTTPResponse response) {
		HTMLDocument doc = this.getServerModule()
				.getMainTemplate(request)
				.addHtml(this.getFile("index.html"));

		// load module states
		Map<String, List<String>> jsModules = new TreeMap<String, List<String>>();

		if (this.module()!=null && this.module().app()!=null) {
			WSNModule[] modules = this.module().app().modules();

			for (WSNModule m: modules) {
				jsModules.put(
						m.getClass().getName(),
						Arrays.asList(new String[]{
								m.getName()!=null ? m.getName() : m.getClass().getName(),
										m.getStatus().getStatus().toString(),
										m.getStatus().getMessage()
						})
						);
			}
		}

		doc.addVar("wsn_driver", this.module()!=null&&this.module().app()!=null&&this.module().app().getDriver()!=null ? this.module().app().getDriver().getName() : "NONE");
		List<String> jsProtocols=new ArrayList<String>();
		if (this.module()!=null && this.module().app()!=null && this.module().app().getProtocolStack()!=null) {
			for (WSNProtocol p: this.module().app().getProtocolStack()) {
				jsProtocols.add(p.getName());
			}
		}
		doc.addVar("wsn_protocols", jsProtocols);

		doc.addVar("module_states", jsModules);


		response.body = doc.toBytes();
	}

	/**
	 * Hardware-Default-Action
	 * @param request
	 * @param response
	 */
	public void hardwareAction(HTTPRequest request, HTTPResponse response) {
		HTMLDocument doc = this.getServerModule()
				.getMainTemplate(request)
				.addHtml(this.getFile("hardware.html"));

		response.body = doc.toBytes();
	}

	/**
	 * Visualisation-Default-Action
	 * @param request
	 * @param response
	 */
	public void visualisationAction(HTTPRequest request, HTTPResponse response) {
		HTMLDocument doc = this.getServerModule()
				.getMainTemplate(request)
				.addHtml(this.getFile("visualisation.html"));

		response.body = doc.toBytes();
	}

	/**
	 * nodes action
	 * 
	 * @param request
	 * @param response
	 */
	public void nodesAction(HTTPRequest request, HTTPResponse response) {
		try {HTMLDocument doc = this.getServerModule()
				.getMainTemplate(request)
				.addHtml(this.getFile("nodes.html"));

		doc.addVar("wsnnodes", this.getServerModule().getNodesVar());
		//doc.addHtml(this.getServerModule().getTemplate("node.html"));
		doc.addHtml(this.getServerModule().getTemplate("topology.html"));

		response.body = doc.toBytes();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * log files
	 * 
	 * @param request
	 * @param response
	 */
	public void logsAction(HTTPRequest request, HTTPResponse response) {
		HTMLDocument doc = this.getServerModule()
				.getMainTemplate(request)
				.addHtml(this.getFile("logs.html"))
				.addVar("log_link", this.module().app().getFilesDirectory());

		response.body = doc.toBytes();
	}
	/**
	 * index action
	 * 
	 * @param request
	 * @param response
	 */
	public void protocolsAction(HTTPRequest request, HTTPResponse response) {
		HTMLDocument doc = this.getServerModule()
				.getMainTemplate(request)
				.addHtml(this.getFile("protocols.html"));
		try {

			doc.addVar("wsnnodes", this.getServerModule().getNodesVar());
			doc.addHtml(this.getServerModule().getTemplate("topology.html"));
			doc.addHtml(this.getServerModule().getTemplate("node.html"));

			response.body = doc.toBytes();
		}catch(Exception e) {e.printStackTrace();}
	}
	/**
	 * index action
	 * 
	 * @param request
	 * @param response
	 */
	public void dynamicAction(HTTPRequest request, HTTPResponse response) {
		HTMLDocument doc = this.getServerModule()
				.getMainTemplate(request)
				.addHtml(this.getFile("dynamic.html"));
		try {

			doc.addVar("wsnnodes", this.getServerModule().getNodesVar());
			doc.addHtml(this.getServerModule().getTemplate("topology.html"));
			doc.addHtml(this.getServerModule().getTemplate("node.html"));

			response.body = doc.toBytes();
		}catch(Exception e) {e.printStackTrace();}
	}


	/**
	 * ajax update action
	 * 
	 * @param request
	 * @param response
	 */
	public void protocolsupdateAction(HTTPRequest request, HTTPResponse response) {
		String log=null;
		if (this.module()!=null && this.module() instanceof WSNHTTPServerModule) {
			WSNHTTPServerModule m = (WSNHTTPServerModule) this.module();
			log = m.getProtocolsLog();
		}


		Map<String,String> jsonResult = new LinkedHashMap<String,String>();
		jsonResult.put("text", log);
		response.body = JSONValue.toJSONString(jsonResult).getBytes();
	}
	/**
	 * ajax update action
	 * 
	 * @param request
	 * @param response
	 */
	public void rawhexupdateAction(HTTPRequest request, HTTPResponse response) {
		String log=null;
		if (this.module()!=null && this.module() instanceof WSNHTTPServerModule) {
			WSNHTTPServerModule m = (WSNHTTPServerModule) this.module();
			log = m.getRawhexLog();
		}


		Map<String,String> jsonResult = new LinkedHashMap<String,String>();
		jsonResult.put("text", log);
		response.body = JSONValue.toJSONString(jsonResult).getBytes();
	}

	/**
	 * Publications
	 * 
	 * @param request
	 * @param response
	 */
	public void publicationsAction(HTTPRequest request, HTTPResponse response) {
		HTMLDocument doc = this.getServerModule()
				.getMainTemplate(request)
				.addHtml(this.getFile("publications.html"));

		response.body = doc.toBytes();
	}


	/**
	 * "under constructoin"
	 * 
	 * @param request
	 * @param response
	 */
	public void underconstructionAction(HTTPRequest request, HTTPResponse response) {
		HTMLDocument doc = this.getServerModule()
				.getMainTemplate(request)
				.addHtml(this.getFile("under_construction.html"));

		response.body = doc.toBytes();
	}

	/**
	 * AJAX Number-Convert-Tool
	 * 
	 * @param request
	 * @param resposne
	 */
	public void numberconverterAction(HTTPRequest request, HTTPResponse response) {
		Map<String,Object> ajaxRet = new HashMap<String,Object>();

		// add entry if requested
		if (request.arguments.containsKey("add")) {
			_numberConverterEntries.put(request.arguments.get("add").get("hex").toString(), true);
		}
		// remove entry if requested
		if (request.arguments.containsKey("rem")) {
			_numberConverterEntries.remove(request.arguments.get("rem").get("hex").toString());
		}

		// set "open" / "closed"
		if (request.arguments.containsKey("status")) {
			_numberConverterOpen = request.arguments.get("status").toString().equals("open") ? true : false;
		}

		response.body = JSONValue.toJSONString(ajaxRet).getBytes();
	}
	protected Map<String,Boolean> _numberConverterEntries = new LinkedHashMap<String,Boolean>();
	protected boolean _numberConverterOpen=false;


	/**
	 * shuts WSN down
	 * 
	 * @param request
	 * @param response
	 */
	public void shutdownAction(HTTPRequest request, HTTPResponse response) {
		HTMLDocument doc = this.getServerModule()
				.getMainTemplate(request)
				.addHtml(this.getFile("shutdown.html"));

		response.body = doc.toBytes();
	}

	public void shutdownajaxAction(HTTPRequest request, HTTPResponse response) {
		if (this.module()!=null && this.module().app()!=null) {
			this.module().app().shutdown();
		}
	}

}
